%% Processing heart rate using simple FFT spectrogram method for AAS conference paper
% Carl Howard 12 August 2018

% This method is useless, as it is not possible to get sufficient
% time-frequency resolution.

%% Heart Signal definitions and limitations
% say that the healthy, expected BPM is between 100 to 200 BPM
% which means the period between beats is t_shift = 60/200 to 60/100 sec
BPM_low=120;
BPM_high=200;


%% Read in the audio file
% The file used in the other analyses:
% heart_beat_filtered_8kHz.wav
% is only 8 kHz, and this does not have enough time-frequency resolution
% Instead, we'll try the original 44100 Hz file.
[y,Fs]=audioread('heart_beat_filtered_8kHz.wav');
%[y,Fs]=audioread('TB16CC_DR0000_0008_020400-022000.wav');



%% Now plot the frequency / heart rate over time.
% Note that the signal must be low pass filtered, to get rid of everything
% above about 3 Hz [ but use BPM_high/60 Hz ].
% Calculate the spectrogram.
% fs must be the fifth input to spectrogram.
%[s,f,t,p] = spectrogram(y2,2^16,[],[],Fs);
figure
[s,f,t,p] = spectrogram(y,2^16,[],[],Fs);
% Look at the first 20 seconds, as that is all I did for the manual
% counting.
%[s,f,t,p] = spectrogram(y2(1:20*Fs),2^16,[],[],Fs);

% Find the frequency limits of f, that correspond to BPM_low and BPM_high
f_index_low=max(find(f<(BPM_low/60)));
f_index_high=min(find(f>(BPM_high/60)));

% Within this frequency range, find where the spectrogram is maximum
[maxval,index]=max(p(f_index_low:f_index_high,:));
time_shift=0.0;
plot(t+time_shift,f(index+f_index_low)*60)
xlabel('Time (s)');
ylabel('Heart Rate (BPM)')
title('Heart Rate calculated using Spectrogram of Audio');
axis([0 360 BPM_low BPM_high]);







%% Band pass filter the data
% This will  band pass filter the date from 50 to 200 Hz
% d = designfilt('bandpassiir','FilterOrder',20, ...
%                 'HalfPowerFrequency1',50,'HalfPowerFrequency2',200, ...
%                 'SampleRate',Fs);
            
% This will  band pass filter the date from 50 to 100 Hz
d = designfilt('bandpassiir','FilterOrder',20, ...
                'HalfPowerFrequency1',50,'HalfPowerFrequency2',100, ...
                'SampleRate',Fs);
          
y_BP=filtfilt(d,y);

time=[1:length(y_BP)].'/Fs;


%%
% Try plotting the envelope using a Hilbert transform
% figure
gain_for_envelope = 10;     % a constant to multiply the audio signal
[up_envelope,~] = envelope((y_BP*gain_for_envelope).^2,4,'peak');

index=find(up_envelope<0);
up_envelope(index)=0;
p1=plot(   time,(y_BP*gain_for_envelope).^2,   ...
        time,up_envelope  );
legend('Filtered 50-100Hz','Envelope');
xlabel('Time (s)');
ylabel('Audio')
title('Envelope Signal')



%% Plot the overall power spectral density spectrum
figure
y2=(y_BP*gain_for_envelope).^2;
pwelch(y2,[],[],[],Fs)



%% Very simple processing method 
% This just uses FFT, nothing fancy.
% Low pass filter below (BPM_high/60) Hz.
%[Blow,Alow] = ellip(2,0.5,40,(BPM_high/60)/(Fs/2),'low'); 	% Low pass filter for demod
%[Blow,Alow] = butter(3,(BPM_high/60)/(Fs/2),'low'); 	% Low pass filter for demod

% You can plot the filter response to make sure it is not doing anything
% weird.
%freqz(Blow,Alow,[],Fs)

% Filter the audio data
%y2= filtfilt(Blow,Alow,y);


%[y2,D_filter] = lowpass(y,(BPM_high/60),Fs,'ImpulseResponse','iir','Steepness',0.999999);
% fvt = fvtool(D_filter);
% legend(fvt,'cqh filter');


% Butterworth Lowpass filter designed using FDESIGN.LOWPASS.

% All frequency values are in Hz.
%Fs = 8000;  % Sampling Frequency

% % All frequency values are in Hz.
% Fs = 44100;  % Sampling Frequency
% 
% Fpass = 5;           % Passband Frequency
% Fstop = 6;           % Stopband Frequency
% Apass = 1;         % Passband Ripple (dB)
% Astop = 30;          % Stopband Attenuation (dB)
% match = 'passband';  % Band to match exactly
% 
% % Construct an FDESIGN object and call its CHEBY1 method.
% h  = fdesign.lowpass(Fpass, Fstop, Apass, Astop, Fs);
% Hd = design(h, 'cheby1', 'MatchExactly', match);
% 
% % View the filter
% fvt = fvtool(Hd);
% axis([0 10e-3 -20 5])


% Filter the audio data
% y2= filter(Hd,y);


%% Plot the overall power spectral density spectrum
% figure
% pwelch(y2,[],[],[],Fs)

%% Now plot the frequency / heart rate over time.
% Note that the signal must be low pass filtered, to get rid of everything
% above about 3 Hz [ but use BPM_high/60 Hz ].
% Calculate the spectrogram.
% fs must be the fifth input to spectrogram.
%[s,f,t,p] = spectrogram(y2,2^16,[],[],Fs);
figure
[s,f,t,p] = spectrogram(y2,2^16,[],[],Fs);
% Look at the first 20 seconds, as that is all I did for the manual
% counting.
%[s,f,t,p] = spectrogram(y2(1:20*Fs),2^16,[],[],Fs);

% Find the frequency limits of f, that correspond to BPM_low and BPM_high
f_index_low=max(find(f<(BPM_low/60)));
f_index_high=min(find(f>(BPM_high/60)));

% Within this frequency range, find where the spectrogram is maximum
[maxval,index]=max(p(f_index_low:f_index_high,:));
time_shift=0.0;
plot(t+time_shift,f(index+f_index_low)*60)
xlabel('Time (s)');
ylabel('Heart Rate (BPM)')
title('Heart Rate calculated using Spectrogram of Envelope');





%% This is the data of manually counting the pulses from the envelope and the raw time size
% the first column is the time where the peak of S1 sound occurs,
% the second column is the BPM, which is calculated as 60/(t_{i}-t_{i-1})
manual_hr2=[0.785	151.8987342
1.17	155.8441558
1.56	153.8461538
1.985	141.1764706
2.405	142.8571429
2.835	139.5348837
3.245	146.3414634
3.62	160
4.02	150
4.4	157.8947368
4.775	160
5.165	153.8461538
5.565	150
5.965	150
6.38	144.5783133
6.795	144.5783133
7.205	146.3414634
7.625	142.8571429
8.005	157.8947368
8.4	151.8987342
8.805	148.1481481
9.215	146.3414634
9.61	151.8987342
10.02	146.3414634
10.44	142.8571429
10.87	139.5348837
11.29	142.8571429
11.71	142.8571429
12.12	146.3414634
12.53	146.3414634
12.9	162.1621622
13.3	150
13.71	146.3414634
14.13	142.8571429
14.54	146.3414634
14.96	142.8571429
15.39	139.5348837
15.83	136.3636364
16.25	142.8571429
16.64	153.8461538
17.05	146.3414634
17.46	146.3414634
17.87	146.3414634
18.27	150
18.69	142.8571429
19.13	136.3636364
19.54	146.3414634
19.98	136.3636364
20.42	136.3636364
];

hold on
pz1=plot(manual_hr2(:,1),manual_hr2(:,2),'--');
legend('FFT','Manual counting')
axis([0 360 BPM_low BPM_high]);
